﻿using UnityEngine;
//using System.Collections;
using System.Collections.Generic;

public class CannonBall : MonoBehaviour {
	
	public float _explosionSize = 2f;
	private float _maxDamage = 60f;
	public AnimationCurve _damageFalloff;
	private Transform _owner;
	private float _ownerGraceTime = 0.5f;
	private float _graceTimer = 0f;
	private List<Transform> _hitPlayers = new List<Transform>();
	private MeshRenderer _renderer;
	private Rigidbody _rb;
	private bool _active = false;

	private void Awake() {
		_renderer = GetComponent<MeshRenderer>();
		_rb = GetComponent<Rigidbody>();
		Disable();
	}

	private void Disable() {
		_active = false;
		_renderer.enabled = false;
		_rb.detectCollisions = false;
		_rb.useGravity = false;
		_rb.velocity = Vector3.zero;
	}

	public void Enable(Transform owner, Vector3 startPos) {
		_active = true;
		transform.position = startPos;
		_owner = owner;
		_graceTimer = _ownerGraceTime;
		_renderer.enabled = true;
		_rb.detectCollisions = true;
		_rb.useGravity = true;
	}

	private void Update() {
		if (_active == true) {
			_graceTimer -= Time.deltaTime;

			if (transform.position.x > 0f && transform.position.x < 64f && transform.position.z > 0f && transform.position.z < 64f) {
				if (CreateLevel.instance.GetDistance(transform.position) <= 0f) {
					Explode();
				}
			}
			else {
				Explode(null, true);
			}
		}
	}

	private void OnTriggerEnter(Collider collider) {
		if (collider.tag == "Player") {
			if (collider.transform == _owner) {
				if (_graceTimer < 0f) {
					Explode(collider.transform);
				}
			}
			else {
				Explode(collider.transform);
			}
		}
		else {
			Explode();
		}
	}

	private void Explode(Transform directHitPlayer = null, bool outside = false) {
		GameManager.instance.ExplosionShake();
		if (outside == false) {
			CreateLevel.instance.AddImpact(new Vector2(transform.position.x, transform.position.z), 0.2F);
		}

		GameManager.instance.PlaySound(GameManager.instance._explosionSound, 0.8f);
		Transform explosion = GameManager.instance.GetExplosion();
		explosion.GetComponent<ExplosionSphere>().Enable(transform.position, _explosionSize);
		
		// Check for direct hit
		if (directHitPlayer != null) {
			directHitPlayer.GetComponent<Tank>().TakeDamage(_maxDamage);
			_hitPlayers.Add(directHitPlayer);
		}
		// Check for area hit
		Collider[] aoeHits = Physics.OverlapSphere(transform.position, _explosionSize / 2f, GameManager.instance._playerMask);
		for (int i = 0; i < aoeHits.Length; i++) {
			if (_hitPlayers.Count == 0 || _hitPlayers.Contains(aoeHits[i].transform) == false) {
				float distance = (aoeHits[i].transform.position - transform.position).magnitude;
				float damage = _maxDamage * _damageFalloff.Evaluate(distance / _explosionSize * 2f);
				aoeHits[i].transform.GetComponent<Tank>().TakeDamage(damage);
			}
		}

		//Destroy(gameObject);
		Disable();
	}
}
