﻿using UnityEngine;
using UnityEngine.SceneManagement;
using System.Collections;
using System.Collections.Generic;

public class GameManager : MonoBehaviour {

	public static GameManager instance;

	public LayerMask _playerMask;
	public bool _slowTurret = false;
	public Transform _ballPrefab;
	public Transform _explosionPrefab;

	public AudioClip _shootSound;
	public AudioClip _explosionSound;
	public AudioClip _playerHitSound;
	public AudioClip _playerDeathSound;

	private List<Tank> _players = new List<Tank>();
	public bool gameOver = false;
	private AudioSource _audioSource;

	private int _poolSize = 10;
	private Transform[] _ballPool;
	private Transform[] _explosionPool;
	private int _ballIndex;
	private int _explosionIndex;
	
	void Awake() {
		instance = this;
		Cursor.lockState = CursorLockMode.Locked;
		_audioSource = GetComponent<AudioSource>();
	}

	private void Start() {
		_ballPool = new Transform[_poolSize];
		_explosionPool = new Transform[_poolSize];
		for (int i = 0; i < _poolSize; i++) {
			Transform ball = Instantiate(_ballPrefab, transform.position, Quaternion.identity, transform.GetChild(0));
			_ballPool[i] = ball;
			Transform explosion = Instantiate(_explosionPrefab, transform.position, Quaternion.identity, transform.GetChild(1));
			_explosionPool[i] = explosion;
		}
	}

	private void Update() {
		if (Input.GetKeyDown(KeyCode.Escape)) {
			Application.Quit();
		}
	}

	public void AddPlayer(Tank player) {
		_players.Add(player);
	}

	public Transform GetBall() {
		Transform ball = _ballPool[_ballIndex];
		_ballIndex++;
		if (_ballIndex == _ballPool.Length) {
			_ballIndex = 0;
		}
		return ball;
	}

	public Transform GetExplosion() {
		Transform explosion = _explosionPool[_explosionIndex];
		_explosionIndex++;
		if (_explosionIndex == _explosionPool.Length) {
			_explosionIndex = 0;
		}
		return explosion;
	}

	public void ExplosionShake() {
		// Check player distance from explosion, strenght based on that
		for (int i = 0; i < _players.Count; i++) {
			_players[i]._camera.CameraShake(0.5f, 0.5f);
		}
		
	}

	public void PlaySound(AudioClip clip, float volume) {
		_audioSource.PlayOneShot(clip, volume);
	}

	public void GameOver() {
		gameOver = true;
		StartCoroutine(PollRestart(1.5F));
	}
	
	IEnumerator PollRestart(float time) {
		yield return new WaitForSeconds(1F);
		if (_players[0]._health == 0F && _players[1]._health == 0F) {
			_players[0]._camera.drawText.gameObject.SetActive(true);
			_players[1]._camera.drawText.gameObject.SetActive(true);
		}
		else if (_players[0]._health == 0F) {
			_players[0]._camera.loserText.gameObject.SetActive(true);
			_players[1]._camera.winnerText.gameObject.SetActive(true);
		}
		else {
			_players[0]._camera.winnerText.gameObject.SetActive(true);
			_players[1]._camera.loserText.gameObject.SetActive(true);
		}
		yield return new WaitForSeconds(time);
		while (true) {
			if (Input.anyKeyDown && gameOver == true) {
				SceneManager.LoadScene(0);
			}
			yield return null;
		}
	}
}
