﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Tank : MonoBehaviour {

	public static float G = 19.62f;

	public int _playerNumber;
	public TankCamera _camera;
	public Transform _bottom;
	public Transform _top;
	public Transform _turretPivot;
	public Transform _shotPoint;
	public Transform _targetObject;
	public AnimationCurve _turningMultp;
	public ParticleSystem _damageParticles;
	public ParticleSystem _deathParticles;
	public TrailRenderer _boostTrail;
	public ParticleSystem _boostParticles;
	public Transform _muzzleFlash;
	public ParticleSystem _shotParticles;

	public AudioClip _idleSoundLoop;
	public AudioClip _movingSoundLoop;
	[HideInInspector]
	public float _health;

	private float _maxSpeed = 7f;
	private float _speed = 0f;
	private float _acceleration = 10f;
	private float _deceleration = 7f;

	private float _turningSpeed = 180f;
	private float _aimTurningSpeed = 3f;
	private float _aimOffset = 35f;

	private float _gravity = 0.3f;
	private Vector3 _gravityVector;
	private float _maxHealth = 100f;
	private float _shotCooldown = 0.5f;
	private float _shotSpeed = 25f;

	private float _maxBoost = 100f;
	private float _boostLeft;
	private float _boostUseRate = 40f;
	private float _boostRechargeCooldown = 1.5f;
	private float _boostAcceleration = 0.5f;
	private float _boostCooldownTimer = 0f;
	private float _boostRechargeRate = 25f;
	private float _boostSpeedMultp = 2f;
	private float _currentBoostMultp;
	private float _boostFowMultp = 0.75f;
	private float _currentFowMultp;
	private bool _boostActive = false;

	private bool _grounded = false;
	private string _controlString;
	private float _cooldownTimer = 0f;
	private AudioSource _audioLoopSource;
	private LineRenderer _lr;
	private Vector3 _collisionVector;
	private float _collisionSpeed;

	private void Awake() {
		GameManager.instance.AddPlayer(this);
		_controlString = "P" + _playerNumber;
		_health = _maxHealth;
		_boostLeft = _maxBoost;

		_camera.Initialize(transform);
		_audioLoopSource = GetComponent<AudioSource>();
		_lr = GetComponent<LineRenderer>();
		_currentFowMultp = _boostFowMultp;
	}

	private void Update() {
		float vertical = Input.GetAxisRaw(_controlString + "Vertical");
		float horizontal = Input.GetAxisRaw(_controlString + "Horizontal");
		// Boost
		if (Input.GetButton(_controlString + "Fire2") && Input.GetAxisRaw(_controlString + "Vertical") >= 0f && _boostLeft > 0f && GameManager.instance.gameOver == false) {
			_boostActive = true;
			_boostLeft -= _boostUseRate * Time.deltaTime;
			_boostCooldownTimer = _boostRechargeCooldown;
			if (_grounded == true) {
				_currentBoostMultp = _boostSpeedMultp;
			}
			else {
				_currentBoostMultp = _boostSpeedMultp * 2f;
			}
			_boostTrail.time = 1f;
			_currentFowMultp = Mathf.Clamp(_currentFowMultp - (Time.deltaTime), _boostFowMultp, 1f);
			_camera.AdjustFow(_currentFowMultp);
			if (_boostParticles.isPlaying == false) {
				_boostParticles.Play();
			}
			if (_boostLeft < 0f) {
				_boostLeft = 0f;
				_boostActive = false;
			}
		}
		else {
			_boostActive = false;
			_boostCooldownTimer -= Time.deltaTime;
			if (_boostCooldownTimer <= 0f) {
				_boostLeft += _boostRechargeRate * Time.deltaTime;
				if (_boostLeft > _maxBoost) {
					_boostLeft = _maxBoost;
				}
			}
			_currentBoostMultp = Mathf.Clamp(_currentBoostMultp - (0.5f * Time.deltaTime), 1f, _boostSpeedMultp);
			_currentFowMultp = Mathf.Clamp(_currentFowMultp + (Time.deltaTime), _boostFowMultp, 1f);
			_camera.AdjustFow(_currentFowMultp);
			_boostTrail.time -= Mathf.Clamp01(5f * Time.deltaTime);
			_boostParticles.Stop();
		}
		_camera.UpdateBoostBar(_boostLeft / _maxBoost);

		// Set speed and rotate
		if (_grounded == true) {
			if (vertical != 0f && GameManager.instance.gameOver == false) {
				if (Mathf.Sign(vertical) == Mathf.Sign(_speed)) {
					_speed = Mathf.Clamp(_speed + _acceleration * vertical * Time.deltaTime, -_maxSpeed * _currentBoostMultp, _maxSpeed * _currentBoostMultp);
				}
				else {
					_speed = Mathf.Clamp(_speed + _acceleration * 2f * vertical * Time.deltaTime, -_maxSpeed * _currentBoostMultp, _maxSpeed * _currentBoostMultp);
				}
			}
			else {
				if (_speed > 0f) {
					_speed = Mathf.Clamp(_speed - _deceleration * Time.deltaTime, 0f, _maxSpeed * _currentBoostMultp);
				}
				else {
					_speed = Mathf.Clamp(_speed + _deceleration * Time.deltaTime, -_maxSpeed * _currentBoostMultp, 0f);
				}
			}
		}
		else if (_boostActive == true) {
			_speed = Mathf.Clamp(_speed + _acceleration * vertical * Time.deltaTime, -_maxSpeed * _currentBoostMultp, _maxSpeed * _currentBoostMultp);
		}
		// Rotate body
		if (_grounded == true) {
			_bottom.Rotate(_bottom.up, _turningSpeed * _turningMultp.Evaluate(Mathf.Abs(_speed) / _maxSpeed) * horizontal * Mathf.Sign(vertical) * Time.deltaTime);
		}
		else {
			_bottom.Rotate(_bottom.up, _turningSpeed * _turningMultp.Evaluate(Mathf.Abs(_speed) / _maxSpeed) * 0.2f * horizontal * Mathf.Sign(vertical) * Time.deltaTime);
		}


		// Move forward & gravity
		transform.Translate(_bottom.forward * _speed * Time.deltaTime, Space.World);
		transform.Translate(_gravityVector, Space.World);
		// Move from collision
		transform.Translate(_collisionVector * _collisionSpeed * Time.deltaTime, Space.World);
		_collisionSpeed = Mathf.Clamp(_collisionSpeed - _deceleration * 2f * Time.deltaTime, 0f, _maxSpeed);

		if (transform.position.x < 1f) {
			transform.position = new Vector3(1f, transform.position.y, transform.position.z);
		}
		else if (transform.position.x > 63f) {
			transform.position = new Vector3(63f, transform.position.y, transform.position.z);
		}
		if (transform.position.z < 1f) {
			transform.position = new Vector3(transform.position.x, transform.position.y, 1f);
		}
		if (transform.position.z > 63f) {
			transform.position = new Vector3(transform.position.x, transform.position.y, 63f);
		}

		float groundHeight = CreateLevel.instance.GetHeight(transform.position);
		if (transform.position.y < groundHeight) {
			//if (_grounded == true) {
			//	_gravityVector += Vector3.one * (groundHeight + transform.position.y);
			//}
			transform.position = new Vector3(transform.position.x, groundHeight, transform.position.z);
			_grounded = true;
		}
		else {
			_grounded = false;
		}
		if (_grounded == true) {
			if (CreateLevel.instance.GetDistance(transform.position + _bottom.forward * 0.75f) < 0.2f) {
				float frontHeight = CreateLevel.instance.GetHeight(transform.position + _bottom.forward * 0.75F);
				float backHeight = CreateLevel.instance.GetHeight(transform.position - _bottom.forward * 0.75F);
				float rightHeight = CreateLevel.instance.GetHeight(transform.position + _bottom.right * 0.5F);
				float leftHeight = CreateLevel.instance.GetHeight(transform.position - _bottom.right * 0.5F);
				float pitchAngle = -Mathf.Atan((frontHeight - backHeight) / 1.5F) * Mathf.Rad2Deg;
				float rollAngle = Mathf.Atan((rightHeight - leftHeight) / 1.5F) * Mathf.Rad2Deg;
				// Debug.Log("frameCount: " + Time.frameCount + "pitchAngle: " + pitchAngle + ", rollAngle" + rollAngle);
				// yawAngle = current yaw
				_bottom.rotation = Quaternion.Euler(pitchAngle, _bottom.rotation.eulerAngles.y, rollAngle);
				//_top.rotation = Quaternion.Euler(pitchAngle, _bottom.rotation.eulerAngles.y, rollAngle);
			}
			_gravityVector = Vector3.zero;
		}
		else {
			_gravityVector -= Vector3.up * _gravity * Time.deltaTime;
			if (_boostActive == true) {
				_bottom.rotation = Quaternion.RotateTowards(_bottom.rotation, Quaternion.Euler(0f, _bottom.rotation.eulerAngles.y, 0f), 10f * Time.deltaTime);
			}
			else {
				_bottom.rotation = Quaternion.RotateTowards(_bottom.rotation, Quaternion.Euler(0f, _bottom.rotation.eulerAngles.y, 0f), 50f * Time.deltaTime);
			}
		}


		// Shooting
		_cooldownTimer -= Time.deltaTime;

		//float startH = _shotPoint.position.y;
		//float angle = _turretPivot.rotation.x;
		//Quaternion.
		//print(angle);
		////angle *= -1f;
		////print(angle);
		//float x = 0f;
		//float arcH = 0f;
		//Vector3 vect = Vector3.zero;
		//List<Vector3> list = new List<Vector3>();
		//for (int i = 0; i < 50; i++) {
		//	x = i * 1f;
		//	arcH = startH + x * Mathf.Tan(angle) - ((G * Mathf.Pow(x, 2)) / (2 * Mathf.Pow(_shotSpeed * Mathf.Cos(angle), 2)));
		//	vect = transform.position + (_top.forward * i);
		//	float groundH = CreateLevel.instance.GetHeight(vect);
		//	//print(i + ": " + arcH + " - " + vect);
		//	list.Add(vect + Vector3.up * arcH);
		//	if (arcH < groundH) {
		//		//print(i + ": " + arcH + " - " + groundH);
		//		_targetObject.position = new Vector3(vect.x, groundH, vect.z);
		//		break;
		//	}
		//	if (i == 49) {
		//		print("SHITTT");
		//	}
		//	//print(i + ": " + height);
		//}
		//_lr.numPositions = list.Count;
		//for (int i = 0; i < list.Count; i++) {
		//	_lr.SetPosition(i, list[i]);
		//}

		//print(_turretPivot.forward);
		if (_boostActive == false) {
			if (Input.GetButtonDown(_controlString + "Fire1") && _cooldownTimer <= 0f && GameManager.instance.gameOver == false) {
				_cooldownTimer = _shotCooldown;
				GameManager.instance.PlaySound(GameManager.instance._shootSound, 1f);
				Transform ball = GameManager.instance.GetBall();
				ball.GetComponent<CannonBall>().Enable(transform, _shotPoint.position);
				ball.GetComponent<Rigidbody>().velocity = _turretPivot.forward * _shotSpeed;
				_camera.CameraShake(0.1f, 0.3f);
				StartCoroutine(MuzzleFlashRoutine());
			}
		}

		// Loop sounds
		if (_speed != 0f) {
			if (_audioLoopSource.clip != _movingSoundLoop) {
				_audioLoopSource.clip = _movingSoundLoop;
				_audioLoopSource.Play();
			}
		}
		else {
			if (_audioLoopSource.clip != _idleSoundLoop) {
				_audioLoopSource.clip = _idleSoundLoop;
				_audioLoopSource.Play();
			}
		}
	}

	private void OnCollisionEnter(Collision collision) {
		if (collision.collider.tag == "Player") {
			collision.collider.GetComponent<Tank>().PlayerCollision(_bottom.forward, _speed);
			_speed = 0f;
		}
	}

	public void PlayerCollision(Vector3 direction, float speed) {
		_collisionVector = direction.normalized;
		_collisionSpeed = speed;
	}

	public void RotateTurret(Quaternion target) {
		// Slow turn
		if (GameManager.instance._slowTurret == true) {
			Quaternion topTarget = Quaternion.Euler(0f, target.eulerAngles.y, 0f);
			_top.rotation = Quaternion.RotateTowards(_top.rotation, topTarget, _aimTurningSpeed / 2f);
			Quaternion turretTarget = Quaternion.Euler(target.eulerAngles.x - _aimOffset, _turretPivot.rotation.eulerAngles.y, 0f);
			_turretPivot.rotation = Quaternion.RotateTowards(_turretPivot.rotation, turretTarget, _aimTurningSpeed / 4f);
		}
		// Instant turn
		else {
			_top.rotation = Quaternion.Euler(0f, target.eulerAngles.y, 0f);
			_turretPivot.rotation = Quaternion.Euler(target.eulerAngles.x - _aimOffset, _turretPivot.rotation.eulerAngles.y, 0f);
		}
	}

	public void TakeDamage(float damage) {
		_health -= damage;
		if (_health < 0f) {
			_health = 0f;
			GameManager.instance.PlaySound(GameManager.instance._playerDeathSound, 1f);
			_deathParticles.Play();
			GameManager.instance.GameOver();
		}
		else {
			GameManager.instance.PlaySound(GameManager.instance._playerHitSound, 1f);
			_damageParticles.Play();
		}
		_camera.UpdateHealthBar(_health / _maxHealth);
	}

	private IEnumerator MuzzleFlashRoutine() {
		//_muzzleFlash.Rotate(Vector3.up, Random.Range(0, 180f));
		_shotParticles.Play();
		_muzzleFlash.gameObject.SetActive(true);
		yield return new WaitForSeconds(0.1f);
		_muzzleFlash.gameObject.SetActive(false);
	}
}
