﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;

public class TankCamera : MonoBehaviour {

	public Image _healthBar;
	public Image _boostBar;
	public Text loserText;
	public Text winnerText;
	public Text drawText;
	
	private Vector2 _currentRot = new Vector2(0f, 0f);
	private Vector2 _sensitivity = new Vector2(160f, 80f);
	private Transform _player;
	private Tank _tankScript;
	private string _controlString;
	private Camera _camera;
	private float _defaultFow;

	private float _distanceFromPlayer = 6f;
	private float _cameraYRotOffset = 40f;
	private Vector3 _playerLookOffset = new Vector3(0f, 1f, 0f);




	public enum ShakeMode { Constant, Smooth, VerySmooth }
	private ShakeMode shakeMode = ShakeMode.Smooth;
	private float shakeStrength = 0.2F;
	private bool cameraShake = false;
	private bool singleFrameShake = false;
	private Vector3 singleFrameShakeDirection = Vector3.zero;
	private float singleFrameShakeStr = 0F;
	private float normalizedShakeTime = 0F;
	private float shakeTime = 0F;
	private float timeFactor = 1F;



	public void Initialize(Transform player) {
		_player = player;
		if (_player.GetComponent<Tank>()._playerNumber == 1) {
			_sensitivity = new Vector2(_sensitivity.x / 2f, _sensitivity.y / 2f);
		}
		_tankScript = _player.GetComponent<Tank>();
		_controlString = "P" + _player.GetComponent<Tank>()._playerNumber;
		_camera = GetComponent<Camera>();
		_defaultFow = _camera.fieldOfView;
	}

	private void Update() {
		_currentRot = new Vector2(_currentRot.x + Input.GetAxis(_controlString + "AimX") * _sensitivity.x * Time.deltaTime, Mathf.Clamp(_currentRot.y - Input.GetAxis(_controlString + "AimY") * _sensitivity.y * Time.deltaTime, -35f, 10f));
	}

	private void LateUpdate() {
		Vector3 direction = new Vector3(0f, 0f, -_distanceFromPlayer);
		Quaternion rotation = Quaternion.Euler(_currentRot.y + _cameraYRotOffset, _currentRot.x, 0f);
		transform.position = _player.position + rotation * direction;
		//print(CreateLevel.instance.GetDistance(transform.position));
		if (transform.position.x > 0f && transform.position.x < 64f && transform.position.z > 0f && transform.position.z < 64f) {
			if (CreateLevel.instance.GetDistance(transform.position) < 0f) {
				float height = CreateLevel.instance.GetHeight(transform.position);
				transform.position = new Vector3(transform.position.x, height + 0.5f, transform.position.z);
			}
		}
		transform.LookAt(_player.position + _playerLookOffset);
		_tankScript.RotateTurret(transform.rotation);

		
		if (cameraShake == true && Time.timeScale != 0) {
			shakeTime -= Time.deltaTime;
			if (shakeTime <= 0f) {
				cameraShake = false;
			}
			normalizedShakeTime += Time.deltaTime * timeFactor;
			float shakeMagnitude;
			Vector3 shakePosition = new Vector3(0, 0, 0);
			if (shakeMode == ShakeMode.Smooth) {
				shakeMagnitude = (1 - ((2 * normalizedShakeTime - 1) * (2 * normalizedShakeTime - 1))) * shakeStrength;
				shakePosition.x += Random.Range(0F, 1F * shakeMagnitude);
				shakePosition.y += Random.Range(0F, 1F * shakeMagnitude);
			}
			else if (shakeMode == ShakeMode.VerySmooth) {
				shakeMagnitude = (Mathf.Pow(2 * normalizedShakeTime - 1, 4) - 2 * Mathf.Pow(2 * normalizedShakeTime - 1, 2) + 1) * shakeStrength;
				shakePosition.x += Random.Range(0F, 1F * shakeMagnitude);
				shakePosition.y += Random.Range(0F, 1F * shakeMagnitude);
			}
			else if (shakeMode == ShakeMode.Constant) {
				shakePosition.x += Random.Range(0F, 1F * shakeStrength);
				shakePosition.y += Random.Range(0F, 1F * shakeStrength);
			}
			transform.position += shakePosition;
		}
		else if (singleFrameShake == true) {
			Vector3 shakePosition = new Vector3(0, 0, 0);
			if (singleFrameShakeDirection == Vector3.zero) {
				shakePosition.x += Random.Range(0F, 1F * singleFrameShakeStr);
				shakePosition.y += Random.Range(0F, 1F * singleFrameShakeStr);
			}
			else {
				shakePosition = singleFrameShakeDirection * singleFrameShakeStr;
			}
			transform.position += shakePosition;
			singleFrameShakeStr = 0F;
		}
	}

	public void UpdateHealthBar(float healthPercentage) {
		_healthBar.fillAmount = healthPercentage;
	}

	public void UpdateBoostBar(float boostPercentage) {
		_boostBar.fillAmount = boostPercentage;
	}

	public void CameraShake(float duration, float strength = 1F, ShakeMode mode = ShakeMode.VerySmooth) {
		cameraShake = true;
		shakeStrength = strength;
		if (shakeTime < duration) {
			shakeTime = duration;
		}
		shakeMode = mode;
		timeFactor = 1F / shakeTime;
		normalizedShakeTime = 0F;
	}

	public void AdjustFow(float multp) {
		_camera.fieldOfView = _defaultFow * multp;
	}

	public void SingleFrameShake(Vector2 direction, float strength) {
		singleFrameShakeDirection = direction;
		if (singleFrameShakeStr < strength) {
			singleFrameShakeStr = strength;
		}
		singleFrameShake = true;
	}
}
